// Global Header file of the CForth LowPower project ************************
//
// Author and (c)2020-24 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.c", "main.h", "cforthLP.c" and
// "cforthLP.h"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger GmbH
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define REVNUM 5			//describes firmware revision

//always STM32F030F6: - LQFP32, 32kbit Flash, 4k SRAM -- only supported HW
//#define LQFP32  always implicitly!!
#define HARDWARE_VERSION 2  //describes project specific hardware version
//#define DEFAULT_LED	//=PB0

//for LowPwr it makes sense only in very special cases:
//#define MOTOR		//ifndef: PWM2 available

#define CLOCK_3M68 //default: HCLCK speed divided by 4 = 921.6kHz
					//SPEED=2:HCLK= 3.6864MHz
					//least supply current single speed mode
//#define CLOCK_14M	//default: HCLCK speed divided by 16 = 921.6kHz
					//SPEED=2:HCLK= 3.6864MHz SPEED=3:HCLK= 14.7456MHz
//#define CLOCK_48M	//special version with 8MHz crystal and 48MHz System Clock
					//else same hardware with RS-232 only. 
					//compared with Std STM32F042:No USB, more I/O, 2 presets
					//option CLOCK_2SPEED:divides 48MHz b 16=3MHz clock speed
					//default is 48MHz

#ifdef CLOCK_48M
#define PLLCLOCK
#endif

// *******************************

//attributes of KernelOperators
//attribute entries are merged with some other functional flags !!
#define A_IME			0x10000 //ATTRIBUTE_IMMEDIATE, cannot be compiled
#define A_CPO			0x20000 //ATTRIBUTE_COMPILEONLY,refuse work interpreting
#define A_BAK			0x40000 //may be parsed in background at executing state
#define A_CI			(A_CPO | A_IME) //COMPILEONLY IMMEDIATE
								//performs complex operations during compilation
#define A_IB			(A_IME | A_BAK)
#define A_RUN			0x80000 //ATTRIBUTE_RUNTIME,
					//compiled by other operators for special runtime behaviour
					//no direct user access, essential for viewing SEE and STEP
//only attribute of UserToken
#define A_VARCON		0x0C000000

#define ERR_STRINGTOKEN 1
#define ERR_SYNTAX		2
#define ERR_STACK		4
#define ERR_COMPONLY	8
#define ERR_SYSTEM		0x10
#define ERR_USERCODE	0x20
#define ERR_CONTEXT 	0x80
#define ERR_USERCONTEXT	0xA0	//combination of both, save code memory
#define ERR_USERSTRING	0x21	//combination of both, save code memory
#define ERR_CANCEL		0x100
#define ERR_RUNTOKEN	0x200
#define ERR_NUMVAL		0x400
#define ERR_DIVZERO		0x800

#define PAGE25 (0x08006400)	//globals: only gBaud, gfNumBase and gProject
#define PAGE26 (0x08006800)	//Project1:single word parameters and gfUserToken
#define PAGE27 (0x08006C00)	//Project1:gfTCode
#define PAGE28 (0x08007000)	//Project2:single word parameters and gfUserToken
#define PAGE29 (0x08007400)	//Project2:gfTCode
#define PAGE30 (0x08007800)	//Project3:single word parameters and gfUserToken
#define PAGE31 (0x08007C00)	//Project3:gfTCode

#define KERNELSIZE		126

#define MAXUSER			63//*4 words p.struct=380 words=1008 bytes=ca.1.0page
#define MAXCODE			0x400	//bytes = 1.0 page

#define MAXDATASTACK	16
#define MAXRETSTACK		16

#define CODETOP         MAXCODE - 1
#define USERTOP         (uint32_t)&gfUserHeader[MAXUSER]

#define USART_RXBUFLEN 452 //512

#define DEFAULT_BAUD	11	//expanded to 115200, 
							//enter first 2 places, e.g. 96 for 9600
#define DEFAULT_NUMBASE	10 //Decimal, enter 16 for HEX

#define CTRL_B  2
#define CTRL_D  4
#define BKSP  8
#define TAB   9
#define LFF  10		//for Macintosh OS: simply exchange values of CRR and LFF
#define CRR  13		//then New_Line will be interpreted correctly
#define CTRL_R  18
#define ESC  27
#define SPACE  32
#define BACKSLASH 92

#endif  //GLOBALDEFINES

// ##########################################################################
// Version changes:
//REVNUM 1: 20 Dec 2023 copied from" "cforth-lp-f042-9.
//			TSSOP20 alternative removed, exclusively LQFP32 supported
//			SRAM relevant features cut down because STM32F030 has only 4 kB
//			For this reason 3 Flash projects are supported with less user code
//			as is MAXUSER->63,MAXCODE->0x400,MAXDATASTACK->16,MAXRETSTACK->16
//			KERNELSIZE->125,  USART_RXBUFLEN->460, FlashPages 27,28,29,30,31
//			kernelOP ARRAY deleted
//20Dec23	bugfix compileTCode: protection against CODETOP overflow
//			Project#3 installed in freed Flash
//21Dec23	CheckValidIO improved
//23Dec23   op_QUEST(): printout of CPUclock improved
//25Dec23	released
//			REVNUM->2
//29Dec23	bugfixes save/load user projects
//			bugfix compilation user header with kernel name (op_colon)
//30Dec23	new gfBackProcess=2: return to higher thread level with CTRL_R
//31Dec23	bugfix gfUserHeader[4] renamed to NSTP (confusion with op_MS)
//released 31Dec23	
//REVNUM->3
//01Jan24	improvement no countdown of NSTP if stepper is not rotating
//			op_MEM now sends block of 128bytes
//			SLEEP is awaked by Hi>Lo pulse at PA3, PA15, PA10(USART RX)
//04Jan24	checkValidIO() and op_PINOH()...op_PINIZ() bugfix and reorg. pinNo
//			pinI/O vs.PWM,MOT check improved
//05Jan24	PWM range at 48M reduced to max 100, max.freq=30kHz
//released 09Jan2024
//REVNUM-->4
//11Jan24	send0X32() modified: always with leading 0x even at small numbers
//18Jan24	bugfix op_SEE()
//19Jan24	decimal numbers can be entered with leading '%' at any muber base
//			see interpretNumber()
//06March24	ADC multiple read 4x average
//09March24	reorg kernel op SAVE and LOAD more Forth typically
//released 14March2024
//REVNUM-->5
//31March24	bugfix cforth.h: DZ and HX have IMMEDIATE flag now
//01Apr24	kernelOp -ROT removed, REGW and REGR implemented new
//02Apr24	doVALUE renamed doVAL, doLIT8B renamed doLIT8
//			bugfix op_SAVEFLASH() and op_LOADFLASH()

// *******************************